#!/usr/bin/env python
# -*- coding: utf-8 -*-
########################################################################
#
# wpsh/wptypes.py
# $Id: wptypes.py,v 1.4 2010/02/10 19:04:14 elias Alpha3 $
#
# wpcmd - A command line WordPress client.
# Copyright (c) 2009 Elias Schwerdtfeger, http://www.tamagothi.de/ 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
########################################################################

"""\
Some types for common objects in the WordPress API with simple functions
for conversion to XMLRPC-structs (hashes).

It is a rather boring piece of code, which should be improved...
"""


import re
import os.path

import webbrowser
import wpshbase



class OutOfDate(wpshbase.Error):
    """Something stored locally is out of date"""
    pass



class PublishError(wpshbase.Error):
    """Something went wrong with the publishing"""
    pass



class ImageSourceMissing(PublishError):
    """The src attribute of the img tag is missing"""
    pass



class AnchorHrefMissing(PublishError):
    """The href attribute of the anchor tag is missing"""
    pass



class UploadFileDoesNotExist(PublishError):
    """The upload file does not exist locally"""
    pass



class CategoryTree(wpshbase.Base):
    """\
    Representation of a category tree.
    
    This class is instantiated normally by BlogConfig and instances
    should not be modified in normal applications.
    """

    def __init__(self, cathash):
        """\
        Constructor.

        Called by BlogConfig, do not construct it by yourself.
        """
        self.__phsh = {}
        # Store the categorys as arrays by parent's ID
        for name in cathash.keys():
            cat = cathash[name]
            p = int(cat.parentId)
            if self.__phsh.has_key(p):
                self.__phsh[p].append(cat)
            else:
                self.__phsh[p] = [cat]
        # Sort each of these arrays alphabetically
        # hihi, looks creepy LISPy with *that* lambda...
        for i in self.__phsh.keys():
            self.__phsh[i].sort(lambda x, y: 
                                cmp(x.categoryName.lower(), 
                                    y.categoryName.lower()))

    def getChildsForId(self, id=0):
        """\
        A more internal function: Get Childs for a category ID.

        If you want to get the top level of the hierarchy, use 
        the ID 0.

        Returns a list of Category instances
        """
        if self.__phsh.has_key(id):
            return self.__phsh[id]
        else:
            return []

    def __printChilds(self, indent, indentadd, parentid):
         for cat in self.getChildsForId(parentid):
             print ' ' * indent + cat.categoryName
             self.__printChilds(indent + indentadd, indentadd, cat.categoryId)

    def printTree(self, indent=4):
        """\
        Print the tree

        indent - Indentation for a sublevel
        """
        self.__printChilds(0, indent, 0)



class Category(wpshbase.Base):
    """\
    A WordPress category.

    COMPONENTS:
    description
    htmlUrl
    categoryDescription
    parentId
    rssUrl
    categoryId
    categoryName
    """

    def __init__(self, hsh=None):
        """\
        Constructor.

        hsh - An optional hash with the WordPress XMLRPC struct
        """
        if hsh:
            self.description = hsh['description']
            self.htmlUrl = hsh['htmlUrl']
            self.categoryDescription = hsh['categoryDescription']
            self.parentId = int(hsh['parentId'])
            self.rssUrl = hsh['rssUrl']
            self.categoryId = int(hsh['categoryId'])
            self.categoryName = hsh['categoryName']
        else:
            self.clear()

    def clear(self):
        """\
        Clear the structure.
        """
        self.description = None
        self.htmlUrl = None
        self.categoryDescription = None
        self.parentId = 0
        self.rssUrl = None
        self.categoryId = 0
        self.categoryName = None

    def visit(self):
        if self.htmlUrl:
            webbrowser.open(self.htmlUrl)
        else:
            raise OutOfDate, "Do a 'config categories' for the blog!"



class ContentBase(wpshbase.Base):
    """\
    The common part of pages and posts.

    COMPONENTS:
    dateCreated
    userid
    description
    title
    link
    permalink
    categories
    excerpt
    text_more
    mt_allow_comments
    mt_allow_pings
    wp_slug
    wp_password
    wp_author

    There is an additional component "encoding" which may be used 
    in later versions for automatic recoding of files with a given
    encoding.
    """
    def __init__(self, hsh=None):
        """\
        Constructor.

        hsh - an optional hash with the XMLRPC struct.
        """
        self.clear()
        if hsh:
            if hsh.has_key('dateCreated'):
                self.dateCreated = hsh['dateCreated']
            if hsh.has_key('userid'):
                self.userid = hsh['userid']
            if hsh.has_key('description'):
                self.description = hsh['description']
            if hsh.has_key('title'):
                self.title = hsh['title']
            if hsh.has_key('link'):
                self.link = hsh['link']
            if hsh.has_key('permalink'):
                self.permalink = hsh['permalink']
            if hsh.has_key('categories'):
                self.categories = hsh['categories']
            if hsh.has_key('excerpt'):
                self.excerpt = hsh['excerpt']
            if hsh.has_key('text_more'):
                self.text_more = hsh['text_more']
            if hsh.has_key('mt_allow_pings'):
                self.mt_allow_pings = hsh['mt_allow_pings']
            if hsh.has_key('mt_allow_comments'):
                self.mt_allow_comments = hsh['mt_allow_comments']
            if hsh.has_key('wp_slug'):
                self.wp_slug = hsh['wp_slug']
            if hsh.has_key('mt_keywords'):
                self.tags = hsh['mt_keywords']

    def clear(self):
        """\
        Clear the structure.
        """
        self.__uploads = {}
        self.__lines = []
        self.__warnings = []
        self.encoding = wpshbase.LOCALENC
        self.dateCreated = None
        self.userid = None
        self.description = None
        self.title = None
        self.link = None
        self.permalink = None
        self.categories = None
        self.excerpt = None
        self.text_more = None
        self.mt_allow_pings = 'open'
        self.mt_allow_comments = 'open'
        self.wp_slug = None
        self.wp_password = None
        self.wp_author = None
        self.tags = None

    def addUpload(self, localpath):
        """\
        Add an upload to the internal structure.

        This is more an internal method, which prepares an upload for
        later processing.

        localpath -- The local path of the upload
        """
        if self.__uploads.has_key(localpath):
            return
        if not os.path.isfile(localpath):
            raise UploadFileDoesNotExist, 'no file %s' % (localpath,)
        self.__uploads[localpath] = None

    # Fixed. Restricting the protocol for four letters breaks https, mailto
    # and gopher. (Okay, anyone remembers gopher?)
    u_re = re.compile(r'[a-z]{4,6}\:\/\/')

    def addPossibleUpload(self, uri):
        """\
        Try to differentiate between local files and internet
        resources.

        uri - The uri or path to check
        """
        # TODO: Bad code!
        if self.u_re.match(uri):
            return
        self.addUpload(uri)

    def getUploadArray(self):
        """Get an array of files to upload."""
        return self.__uploads.keys()

    def registerUploadedPath(self, local, remote):
        """\
        Register the path of an uploaded file.

        local - local filename
        remote - uploaded path
        """
        self.__uploads[local] = remote

    def replaceUploadUris(self):
        """\
        Replace the upload filepaths with the uploaded filepaths
        """
        self.description = self.__replaceUploadUrisPart(self.description)
        self.text_more = self.__replaceUploadUrisPart(self.text_more)
        self.excerpt = self.__replaceUploadUrisPart(self.excerpt)

    def __replaceUploadUrisPart(self, txt):
        # TODO: Bad code!
        if not txt:
            return None
        for i in self.__uploads.keys():
            txt = txt.replace(i, self.__uploads[i])
        return txt

    def getXMLRPCHash(self):
        """\
        Get the hash for XMLRPC upload.

        Returns the hash
        """
        # The categories are an array in the struct, so I have to
        # create them as array.
        ca = [i.strip() for i in self.categories.split(',')]
        h = {}
        if self.dateCreated:
            h['dateCreated'] = self.dateCreated
        if self.userid:
            h['userid'] = self.userid
        if self.description:
            h['description'] = self.description
        if self.title:
            h['title'] = self.title
        if self.link:
            h['link'] = self.link
        if self.permalink:
            h['permalink'] = self.permalink
        if self.categories:
            h['categories'] = ca
        if self.excerpt:
            h['mt_excerpt'] = self.excerpt
        if self.text_more:
            h['mt_text_more'] = self.text_more
        if self.mt_allow_pings:
            h['mt_allow_pings'] = self.mt_allow_pings
        if self.mt_allow_comments:
            h['mt_allow_comments'] = self.mt_allow_comments
        if self.wp_slug:
            h['wp_slug'] = self.wp_slug
        if self.tags:
            h['mt_keywords'] = self.tags
        return h

    def getMetaLines(self):
        """\
        Get meta lines for a text.

        Returns the Lines as an array.
        This is only for future functionality.

        PLUGIN SLOTS:
        GetMetaLines - To filter or change the lines
        """
        raise NotImplementedError, 'For future functionality'

    def feed(self, metaPart, inputParser):
        """\
        Feed the instance from parsed input.

        metaPart - The MetaPart instance
        inputParser - The InputParser instance
        """
        self.clear()
        self.description = inputParser.getText()
        self.title = metaPart.getTitle()
        self.tags = metaPart.getMetaCommaSeparated('tags')
        self.__parseUploads()

    def __parseUploads(self):
        self.__parseUploadsPart(self.description)
        self.__parseUploadsPart(self.excerpt)
        self.__parseUploadsPart(self.text_more)

    # RE to scan for a tag in a general way, finds all tags
    tag_re = re.compile(r'\<(\w+)[^\>]*\>')

    # RE to scan for the src attribute
    # It fails if the filename contains quotation marks.
    src_re = re.compile(r'src\=[\'"]([^\'"]+)[\'"]', re.I)

    # RE to scan for the href attribute
    # It fails if the filename contains quotation marks.
    href_re = re.compile(r'href\=[\'"]([^\'"]+)[\'"]', re.I)

    def __parseUploadsPart(self, txt):
        if not txt:
            return
        for tagmatch in self.tag_re.finditer(txt):
            tagname = tagmatch.group(1).lower()
            fulltag = tagmatch.group(0)
            if tagname == 'img':
                mo = self.src_re.search(fulltag)
                if not mo:
                    raise ImageSourceMissing, 'No src for image'
                self.addPossibleUpload(mo.group(1))
            elif tagname == 'a':
                mo = self.href_re.search(fulltag)
                if not mo:
                    raise AnchorHrefMissing, 'No href for anchor'
                self.addPossibleUpload(mo.group(1))



class Page(ContentBase):
    """\
    A WordPress Page.

    ADDITIONAL COMPONENTS:
    page_id
    page_parent_id
    page_status
    page_order
    wp_page_template
    """

    def __init__(self, hsh=None):
        """\
        Constructor.

        hsh - An optional hash with the struct.
        """
        ContentBase.__init__(self, hsh)
        if hsh:
            if hsh.has_key('page_id'):
                self.page_id = int(hsh['page_id'])
            if hsh.has_key('page_parent_id'):
                self.page_parent_id = hsh['page_parent_id']
            if hsh.has_key('page_status'):
                self.page_status = hsh['page_status']
            if hsh.has_key('wp_page_template'):
                self.wp_page_template = hsh['wp_page_template']
        
    def clear(self):
        ContentBase.clear(self)
        self.page_id = None
        self.page_status = 'publish' # We wanna publish by default!
        self.page_parent_id = None
        self.page_order = None
        self.wp_page_template = None

    def getXMLRPCHash(self):
        h = ContentBase.getXMLRPCHash(self)
        h['post_type'] = 'page'
        if self.page_id:
            h['page_id'] = self.page_id
        if self.page_status:
            h['page_status'] = self.page_status
        if self.page_parent_id:
            h['page_parent_id'] = self.page_parent_id
        if self.page_order:
            h['page_order'] = self.page_order
        if self.wp_page_template:
            h['wp_page_template'] = self.wp_page_template
        return h



class Post(ContentBase):
    """\
    A WordPress Posting.

    ADDITIONAL COMPONENTS:
    post_id
    post_status
    categories
    trackback
    """

    def __init__(self, hsh=None):
        """\
        Constructor.

        hsh - An optional hash with the struct.
        """
        ContentBase.__init__(self, hsh)
        if hsh:
            pass

    def clear(self):
        ContentBase.clear(self)
        self.post_id = None
        self.post_status = 'publish'
        self.categories = None
        self.trackback = None

    def getXMLRPCHash(self):
        h = ContentBase.getXMLRPCHash(self)
        h['post_type'] = 'post'
        if self.post_id:
            h['post_id'] = self.post_id
        if self.post_status:
            h['post_status'] = self.post_status
        if self.trackback:
            h['mt_tb_ping_urls'] = self.trackback
        return h

    def feed(self, metaPart, inputParser):
        ContentBase.feed(self, metaPart, inputParser)
        self.categories = metaPart.getMetaCommaSeparated('categories')




########################################################################
#
# $Log: wptypes.py,v $
# Revision 1.4  2010/02/10 19:04:14  elias
# Fixed bugs in upload file parsing.
# Fixed bug in protocol spec parsing.
#
# Revision 1.3  2009/09/23 15:59:39  elias
# Source cleanup for Alpha2
#
# Revision 1.2  2009/09/20 20:16:43  elias
# Source cleanup for alpha release.
#
# Revision 1.1  2009/09/20 02:21:33  elias
# Initial revision
#
########################################################################
#
# CODE IS PROSA, NOT POETRY!
#
########################################################################
